<?
function dir_normalize($path)
{
  // Do path mangling and return both the real and virtual location
  // of the specified path.

  global $DOCUMENT_ROOT;
  $DOCUMENT_ROOT = realpath($DOCUMENT_ROOT);

  $orig = $path;

  // Allow absolute links starting from wwwroot
  if(substr($path, 0, 1) == "/")
  {
    $path = $DOCUMENT_ROOT . $orig;
  }
  else
  {
    $path = realpath($orig);
  }

  $wwwpath = str_replace($DOCUMENT_ROOT, "", $path);

  // This usually indicates that a file is symlinked outside wwwroot
  if($wwwpath == $path)
  {
    // I'm not sure what the correct solution is;
    // it just reverts to relative links, which seems okay.
    $wwwpath = $orig;
    $wwwpath = str_replace("/./", "/", $wwwpath);
  }

  if(!$path) return array($orig, $orig);
  return array($wwwpath, $path);
}

function remove_args($url)
{
  // Converts "page.php?foo=bar" to "page.php"
  $index = strpos($url, "?");
  if ($index  and  $index >= 0)
  {
    $text = substr($url, 0, $index);
    $url = $text;
  }

  return $url;
}

function search_parents($filename, $directory="")
{
  // Provides "inheritance" in a somewhat object-oriented manner.
  // Similar to what Zope does.
  global $inherit_path;

  // allow links to pass GET parameters, but ignore them for now
  $fixed_filename = remove_args($filename);

  // Allow absolute addresses.  Don't use inheritance for hardwired paths.
  if(substr($filename, 0, 1) == "/")
  {
    $check = dir_normalize($filename);
    return $check;
  }

  // Now, search through the parents until something is found.
  foreach($inherit_path as $path)
  {
    $check = $path . "/" . $fixed_filename;
    $urlcheck = $path . "/" . $filename;
    if (file_exists($check))
    {
      $check = dir_normalize($urlcheck);
      return $check;  // found something
    }
    // Search in subdirectories, if told to do so.
    if ($directory)
    {
      $check = $path . "/" . $directory . "/" . $fixed_filename;
      $urlcheck = $path . "/" . $directory . "/" . $filename;
      if (file_exists($check))
      {
	$check = dir_normalize($urlcheck);
	return $check; // found something
      }
    }
  }
  return "";  // fail
}

function image($file, $alt="", $extra="")
{
  // Automatically add wid/hgt (and other) tags to images...
  $stuff = search_parents($file, "gfx");
  if(!$stuff) return;

  $file = $stuff[0];
  $realfile = $stuff[1];

  // Generate alt tags based on image filename
  if($alt == "") { $alt = basename($realfile); }

  // get width and height info
  $size = GetImageSize($realfile);
  // produce a detailed <img /> tag
  $text = sprintf("<img src=\"%s\" alt=\"%s\" %s %s border=\"0\" />",
		  $file, $alt, $extra, $size[3]);

  echo $text;
}

function bg_image($file)
{
  // Used inside a <body> or <table> tag, instead of background="foo.png"
  $stuff = search_parents($file, "gfx");
  if(!$stuff) return;

  $file = $stuff[0];
  $realfile = $stuff[1];

  printf("background=\"%s\"", $file);
}

function page($file, $text="", $extra="")
{
   print make_page($file, $text, $extra);
}

function make_page($file, $text="", $extra="")
{
  // Link to another page on this site.

  // ... or link to a different site.
  if("http" == substr($file, 0, 4) )
  {
    return "<a href=\"{$file}\" $extra>{$text}</a>";
  }

  $stuff = search_parents($file, "");
  if(!$stuff) return;

  $file = $stuff[0];
  $realfile = $stuff[1];

  // use the link as its own description, if none is given
  if(! $text) { $text = $file; }

  return "<a href=\"{$file}\" {$extra}>{$text}</a>";
}

function href($file, $text="", $extra="")
{
  // Link to another page on this site.
  // I wanted to call it file(), but php already claimed that name.
  $stuff = search_parents($file, "files");
  if(!$stuff) return;

  if(!$text) {$text = $file;}
  $file = $stuff[0];
  $realfile = $stuff[1];

  printf("<a href=\"%s\" $extra>%s</a>", $file, $text);
}
?>
